/* Nessus
 * Copyright (C) 1998 - 2003 Renaud Deraison
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * In addition, as a special exception, Renaud Deraison
 * gives permission to link the code of this program with any
 * version of the OpenSSL library which is distributed under a
 * license identical to that listed in the included COPYING.OpenSSL
 * file, and distribute linked combinations including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * this file, you may extend this exception to your version of the
 * file, but you are not obligated to do so.  If you do not wish to
 * do so, delete this exception statement from your version.
 * 
 * Plugins requirements
 *
 */ 
 
#include <includes.h>
#include "plugs_req.h"

/**********************************************************
 
 		   Private Functions
	
***********************************************************/
 
/*---------------------------------------------------------

  Returns whether a port in a port list is closed or not
 
 ----------------------------------------------------------*/
static int
get_closed_ports(data, keys, ports, preferences)
   struct arglist * data;
   struct arglist * keys;
   struct arglist * ports;
   struct arglist * preferences;
{

  if(ports == NULL || data == NULL)
   return -1;
  
  if(arg_get_value(data, "key") == NULL)
    arg_add_value(data, "key", ARG_ARGLIST, -1, keys);
    
  if(arg_get_value(data, "preferences") == NULL)
    arg_add_value(data, "preferences", ARG_ARGLIST, -1, preferences);
  
  
  while(ports->next != NULL)
  {
   int iport = atoi(ports->name);			
   if(iport != 0)
   	{
      	if( host_get_port_state(data, iport) != 0 )
		return iport;
	}
      else 
        {
      	if( arg_get_value(keys, ports->name) != NULL )
		return 1; /* should be the actual value indeed ! */
	}   
    ports = ports->next;
  }
  return 0; /* found nothing */
}


/*-----------------------------------------------------------

  Returns whether a port in a port list is closed or not
 
 ------------------------------------------------------------*/
static int
get_closed_udp_ports(data, keys, ports, preferences)
   struct arglist * data;
   struct arglist * keys;
   struct arglist * ports;
   struct arglist * preferences;
{
  if(arg_get_value(data, "key") == NULL)
   arg_add_value(data, "key", ARG_ARGLIST, -1, keys);
  
  if(arg_get_value(data, "preferences") == NULL)
   arg_add_value(data, "preferences", ARG_ARGLIST, -1, preferences);
   
  if(ports == NULL || data == NULL)
  	return -1;
  else while( ports->next != NULL)
  {
      int iport = atoi(ports->name);				
      if(host_get_port_state_udp(data, iport))return iport;
      ports = ports->next;
  }
  return 0; /* found nothing */
}

/*-----------------------------------------------------------
            
	     Returns the set of required ports
	    
 -----------------------------------------------------------*/
 
static struct arglist *
get_required_ports(plugin)
   struct arglist * plugin;
{
  struct arglist * args = arg_get_value(plugin, "plugin_args");
  if( args != NULL )return arg_get_value(args, "required_ports");
  else return NULL;
}


/*-----------------------------------------------------------
            
	     Returns the set of required UDP ports
	    
 -----------------------------------------------------------*/
static struct arglist *
get_required_udp_ports(plugin)
   struct arglist * plugin;
{
  struct arglist * args = arg_get_value(plugin, "plugin_args");
  if( args != NULL )return arg_get_value(args, "required_udp_ports");
  else return NULL;
}


/*-----------------------------------------------------------
            
	     Returns the set of required keys
	    
 -----------------------------------------------------------*/
static struct arglist *
get_required_keys(plugin)
 struct arglist * plugin;
{
 struct arglist * args = arg_get_value(plugin, "plugin_args");
 if(args)return arg_get_value(args, "required_keys");
 else return NULL;
}


/*-----------------------------------------------------------
            
	     Returns the set of excluded keys
	    
 -----------------------------------------------------------*/
static struct arglist *
get_excluded_keys(plugin)
 struct arglist * plugin;
{
 struct arglist * args = arg_get_value(plugin, "plugin_args");
 if(args)return arg_get_value(args, "excluded_keys");
 else return NULL;
}

/*-----------------------------------------------------------
            
	     Returns the name of the first key
	     which is not in <keyring>
	    
 -----------------------------------------------------------*/
static char * 
key_missing(keyring, keys)
  struct arglist * keyring;
  struct arglist * keys;
{
 if(!keyring || !keys)return NULL;
 else {
   while(keys && keys->next)
   {
     if(!arg_get_value(keyring, keys->value))return keys->value;
     keys = keys->next;
   }
 }
 return NULL;
}

/*-----------------------------------------------------------
            
	    The opposite of the previous function
	    
 -----------------------------------------------------------*/
static char * key_present(keyring, keys)
 struct arglist * keyring;
 struct arglist * keys;
{
 if(!keyring || !keys)return NULL;
 else {
 while(keys && keys->next)
 {
  if(arg_get_value(keyring, keys->value))return keys->value;
  keys = keys->next;
 }
 return NULL;
 }
} 

/**********************************************************
 
 		   Public Functions
	
***********************************************************/	




/*------------------------------------------------------

  Returns <port> if the lists of the required ports between
  plugin 1 and plugin 2 have at least one port in common
 
 
 ------------------------------------------------------*/
struct arglist * 
requirements_common_ports(plugin1, plugin2)
 struct arglist * plugin1, *plugin2;
{
 struct arglist * ret = NULL;
 struct arglist * req1; 
 struct arglist * req2;
 
 if(!plugin1 || !plugin2) return 0;
 
 req1 = get_required_ports(plugin1);
 req2 = get_required_ports(plugin2);
 if(!req1 || !req2)return 0;
 while(req1 && req1->next)
 {
  struct arglist * r = req2;
  while(r && r->next)
  {
   if(req1->type == r->type)
   {
      if(r->name && req1->name && !strcmp(r->name, req1->name))
       {
       if(!ret)ret = emalloc(sizeof(struct arglist));
       arg_add_value(ret, r->name, ARG_INT, 0,(void*)1);
       }
   }  
   r = r->next;
  }
  req1 = req1->next;
 }
 return ret;
}


/*-------------------------------------------------------

	Determine if the plugin requirements are
	met.

	Returns NULL is everything is ok, or else
	returns an error message

---------------------------------------------------------*/

char *
requirements_plugin(kb, plugin, portsinfos, preferences)
 struct arglist * kb;
 struct arglist * plugin;
 struct arglist * portsinfos;
 struct arglist * preferences;
{
  char * missing;
  char * present;
  struct arglist * tcp, * udp;
  /*
   * Check wether the good ports are open
   */
  tcp = get_required_ports(plugin->value);
 
  
  if(tcp != NULL && (get_closed_ports(portsinfos, kb, tcp , preferences)) == 0)
     {
      char * error = strdup("none of the required tcp ports are open");
      return error;
     }
      
   udp = get_required_udp_ports(plugin->value);  
   if(udp != NULL && (get_closed_udp_ports(portsinfos, kb, udp , preferences)) == 0)
      {
      char * error = strdup("none of the required udp ports are open");
      return error;
      }
  
  /*
   * Check wether a key we wanted is missing
   */
  if((missing = key_missing(kb, get_required_keys(plugin->value))))
  {
     char * error = emalloc(strlen(missing) + 30);
     sprintf(error, "because the key %s is missing", missing);
     return error;
  }
  
  /*
   * Check wether a plugin we do not want is present
   */
  if((present = key_present(kb, get_excluded_keys(plugin->value))))
  {
   char * error = emalloc(strlen(present) + 30);
   sprintf(error, "because the key %s is present", present);
   return error;
  }
  return NULL;
}
